""" Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import time
import json

from utils.rabbitmq import RabbitMQPublisher
from connectors.core.constants import *
from django.conf import settings

import logging

logger = logging.getLogger("connectors")


def publish_audit_log(data, exchange_name="texchange.cyops.sdk", routing_key="key.sdk.audit", retry_count=settings.PUBLISH_MESSAGE_RETRY_COUNT):
    rmq = RabbitMQPublisher()
    rmq.publish(exchange_name,
                routing_key,
                body=json.dumps(data),
                content_type="application/json",
                retry_count=retry_count
                )


def publish_audit_and_notify(data, method, module, exchange, routing_key, title="", request=None):
    try:
        if settings.LW_AGENT:
            return
        operation = 'Create' if method == 'POST' else 'Update' if method == 'PUT' else 'Delete' if method == 'DELETE' else method
        user_name = ""
        user_uuid = ""
        record_uuid = ""
        if request:
            user_uuid = request.data.get('rbac_info', {}).get('user_iri')
            user_uuid = user_uuid.split('/')[-1] if user_uuid else None
            source = request.data.get('rbac_info', {}).get('source', settings.SELF_ID)
            record_iri = request.data.get('audit_info', {}).get('record_iri')
            module = record_iri.split('/')[-2] if record_iri else module
            record_uuid = record_iri.split('/')[-1] if record_iri else ""

        else:
            user_name = "Master"
            source = settings.SELF_ID
            if settings.MASTER_ID:
                user_name = "Agent"
        if user_uuid is None:
            return
        audit_data = {
            "data": data,  # Format for data {'data':data, 'message': message, 'status': status}
            "operation": operation,
            "source": source,
            "component": "agent",
            "entityType": module.lower(),
            "entitySingularName": "" if record_uuid else module.rstrip('s'),
            "entityUuid": record_uuid,
            "userId": user_uuid,
            "user": user_name,
            'transactionDate': int(round(time.time() * 1000)),
            'title': title
        }
        publish_audit_log([audit_data], exchange_name=exchange, routing_key=routing_key, retry_count=1)

    except Exception as e:
        logger.error("Error while auditing connector action execution %s", str(e))


def audit_connector_action(audit_data, status):
    try:
        if settings.LW_AGENT:
            return
        user_iri = audit_data.get("user_iri")
        user_id = user_iri.split('/')[-1] if user_iri else ""

        record_iri = audit_data.get("record_iri")
        record_type = record_iri.split('/')[-2] if record_iri else ""
        record_uuid = record_iri.split('/')[-1] if record_iri else ""
        audit_data['data']['status'] = status

        audit_data = {
            "operation": CONNECTOR_AUDIT_OPERATIONS.get("execute"),
            "component": "sdk",
            "entityType": record_type,
            "entityUuid": record_uuid,
            "userId": user_id,
            "source": audit_data.get("source"),
            "transactionDate": int(round(time.time() * 1000)),
            "data": audit_data['data']
        }
        publish_audit_log([audit_data])
    except Exception as e:
        logger.error("Error while auditing connector action execution %s", str(e))
